#include <stdio.h>
#include "arg.h"
#include "log.h"
#include "dep.h"
#include "db.h"
#include "tmpasm_scconfig.h"
#include <string.h>

/* Runs when a custom command line argument is found
 returns true if no furhter argument processing should be done */
int hook_custom_arg(const char *key, const char *value)
{
	if(0==strcmp(key, "levelsdir"))
	{
		put("/local/ncbricks/levelsdir", value);
		return 1;
	}

	return 0;
}


/* Runs before anything else */
int hook_preinit()
{
	return 0;
}

/* Runs after initialization */
int hook_postinit()
{
	db_mkdir("/local");
	put("/local/ncbricks/debug", sfalse);
	put("/local/ncbricks/levelsdir", "/usr/share/ncbricks/levels");
	return 0;
}

/* Runs after all arguments are read and parsed */
int hook_postarg()
{
	return 0;
}

/* Runs when things should be detected for the host system */
int hook_detect_host()
{
	return 0;
}

/* Runs when things should be detected for the target system */
int hook_detect_target()
{
	if(istrue(get("/local/ncbricks/debug")))
	{
		require("cc/argstd/Wall", 0, 0);
	}

	require("cc/cc", 0, 0);
	require("libs/tty/ncurses/cflags", 0, 0);
	require("libs/tty/ncurses/includes", 0, 0);
	require("libs/tty/ncurses/ldflags", 0, 0);
	return 0;
}

/* Runs after detection hooks, should generate the output (Makefiles, etc.) */
int hook_generate()
{
	const char* const root = "../src";
	int generr = 0;

	fprintf(stderr, "Generating libvmma/Makefile (%d)\n", generr |= tmpasm(root, "Makefile.in", "Makefile"));
	fprintf(stderr, "Generating libvmma/config.h (%d)\n", generr |= tmpasm(root, "config.h.in", "config.h"));

	return generr;
}

/* Runs before everything is uninitialized */
void hook_preuninit()
{
}

/* Runs at the very end, when everything is already uninitialized */
void hook_postuninit()
{
}

