/*
Copyright 2017-2018 Aron Barath

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors
may be used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#include "ncbricks.h"
#include <stdlib.h>
#include <string.h>
#include <assert.h>

static void charm_random();

static void charm_cash()
{
	add_score(SCORE_EXTRA);
}

static void charm_life()
{
	add_life(1);
}

static void charm_ammo1()
{
	add_ammo(AMMO_EXTRA1);
}

static void charm_ammo2()
{
	add_ammo(AMMO_EXTRA2);
}

static void charm_paddle_inc()
{
	paddle_enlarge(&g_paddle, +1);
}

static void charm_paddle_dec()
{
	paddle_enlarge(&g_paddle, -1);
}

static const struct charmdesc charms[] =
{
	/* this must be the first! */
	{ .symbol = '?', .desc = "Random charm event", .effect = charm_random },
	/* other charms go here */
	{ .symbol = '$', .desc = "Earn extra points", .effect = charm_cash },
	{ .symbol = '*', .desc = "Enable atomic ball for a short time", .effect = enable_atomic_ball },
	{ .symbol = '&', .desc = "Earn an additional ball", .effect = charm_life },
	{ .symbol = ':', .desc = "Small ammo pack", .effect = charm_ammo1 },
	{ .symbol = ';', .desc = "Large ammo pack", .effect = charm_ammo2 },
	{ .symbol = '=', .desc = "Split all balls on the board", .effect = ball_split_all },
	{ .symbol = '#', .desc = "Missile apocalypse", .effect = missile_spawn_apocalypse },
	{ .symbol = '-', .desc = "Make the paddle narrower", .effect = charm_paddle_dec },
	{ .symbol = '+', .desc = "Make the paddle wider", .effect = charm_paddle_inc },
	{ .symbol = '^', .desc = "Kill all balls", .effect = ball_delete_all },
	{ .symbol = '@', .desc = "Slow ball", .effect = enable_slow_ball }
};

#define N_CHARMS (sizeof(charms)/sizeof(charms[0]))

static void charm_random()
{
	const size_t i = 1 + (rand() % (N_CHARMS-1));
	charms[i].effect();
}

void charm_show_all()
{
	size_t i;

	for(i=0;i<g_charms.size;++i)
	{
		charm_show((struct charm*)g_charms.pointers[i]);
	}
}

void charm_hide_all()
{
	size_t i;

	for(i=0;i<g_charms.size;++i)
	{
		charm_hide((struct charm*)g_charms.pointers[i]);
	}
}

void charm_show(const struct charm* const charm)
{
	if(g_use_colors)
	{
		wattron(g_game_win, COLOR_PAIR(CPAIR_CHARM));
	}

	mvwaddch(g_game_win, g_board_top+charm->pos.y, g_board_left+charm->pos.x, charm->charm|A_BOLD);

	if(g_use_colors)
	{
		wattroff(g_game_win, COLOR_PAIR(CPAIR_CHARM));
	}
}

void charm_hide(const struct charm* const charm)
{
	mvwaddch(g_game_win, g_board_top+charm->pos.y, g_board_left+charm->pos.x, ' ');
}

void charm_delete_all()
{
	while(0<g_charms.size)
	{
		struct charm* const charm = (struct charm*)g_charms.pointers[--g_charms.size];
		charm_hide(charm);
		free(charm);
	}
}

static const struct charmdesc* lookup(const char typ)
{
	size_t i;

	for(i=0;i<N_CHARMS;++i)
	{
		if(typ==charms[i].symbol)
		{
			return charms + i;
		}
	}

	return 0;
}

void charm_spawn(const struct brick* const brick, const char typ)
{
	struct charm* const charm = (struct charm*)malloc(sizeof(struct charm));
	assert(NULL!=charm);
	memset(charm, 0, sizeof(struct charm));

	charm->pos.x  = (brick->pos.pos.x + brick->pos.end.x) / 2;
	charm->pos.y  = brick->pos.end.y;
	charm->charm  = typ;
	charm->effect = lookup(typ)->effect;
	ptrarr_add(&g_charms, charm);
}

char charm_decode(const char charm)
{
	const struct charmdesc* const desc = lookup(charm);

	if(NULL!=desc)
	{
		return charm;
	}

	return 0;
}
