/*
Copyright 2017-2018 Aron Barath

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

3. Neither the name of the copyright holder nor the names of its contributors
may be used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
*/

#include "ptrarr.h"
#include <stdlib.h>
#include <string.h>

#define DEFAULT_CAPACITY 16

void ptrarr_init(struct ptrarr* const arr)
{
	memset(arr, 0, sizeof(*arr));

	if(NULL==(arr->pointers=(void**)malloc(sizeof(void*)*DEFAULT_CAPACITY)))
	{
		abort();
	}

	arr->capacity = DEFAULT_CAPACITY;
}

void ptrarr_add(struct ptrarr* const arr, void* const ptr)
{
	if(arr->size==arr->capacity)
	{
		if(NULL==(arr->pointers=(void**)realloc(arr->pointers, 2*sizeof(void*)*arr->capacity)))
		{
			abort();
		}

		arr->capacity *= 2;
	}

	arr->pointers[arr->size++] = ptr;
}

void ptrarr_remove(struct ptrarr* const arr, size_t index)
{
	/* sanity check -- maybe a little bit too harsh, but who cares */
	if(index>=arr->size) { abort(); }

	if((index+1)<arr->size)
	{
		arr->pointers[index] = arr->pointers[arr->size-1];
	}

	--arr->size;
}
